@@ -3,7 +3,6 @@ import envi.common as e_common
 import vivisect.parsers as viv_parsers
 import vstruct.defs.macho as vs_macho
 
-
 def parseFile(vw, filename, baseaddr=None):
     with open(filename, 'rb') as f:
         fbytes = f.read()
@@ -14,6 +13,11 @@ def parseBytes(vw, filebytes, baseaddr=None):
     return _loadMacho(vw, filebytes, baseaddr=baseaddr)
 
 
+def parseMemory(vw, memobj, baseaddr):
+    byts = memobj.read()
+    return _loadMacho(vw, byts, baseaddr=baseaddr)
+
+
 archcalls = {
     'i386': 'cdecl',
     'amd64': 'sysvamd64call',
@@ -99,8 +103,3 @@ def _loadMacho(vw, filebytes, filename=None, baseaddr=None):
         vw.addLibraryDependancy(libname)
 
     return fname
-
-
-def parseMemory(vw, memobj, baseaddr):
-    # TODO: implement
-    pass
